﻿namespace Example.Application.Tests.Services
{
    using Example.Application.Services;
    using Microsoft.Extensions.Configuration;
    using System;
    using System.Collections.Generic;
    using Xunit;

    /// <summary>
    /// The key service tests.
    /// </summary>
    public class KeyServiceTests
    {
        #region Private Fields

        /// <summary>
        /// The enigma config.
        /// </summary>
        private static readonly Dictionary<string, string> EnigmaConfig = new Dictionary<string, string>
                                                                              {
                                                                                  {
                                                                                      "Enigma:PublicKey",
                                                                                      "020690B756D7B03CB420D07A85B26850A5033327M3GL3LVMYW7MR43MWQSTWQXNNDEZ7"
                                                                                      + "2CKGWZUBESXF22V2PWSWZXY3TY24PVZJKXHP97ZUYFJMKT2N8AYVCFM6ASK2N9KRM6X"
                                                                                      + "2Z44JFFGS7YWNC2LA8ZYBZQ34295UD2FX6KRT3U7MY9QKDN9TGZESXHZCQYBWN8P87H"
                                                                                      + "BHW68NTUPUAFXVV57882AFUF5Q7B6X6X9CA5XMCD6AJTJR9DEZX62F3GN6CVZ4FASKR"
                                                                                      + "E8T9M3KPFGZWBS8WHCTNR585YUV55DWZCFRDW8YVPUN5YRABETG7PSYSZHW46B3PT3J"
                                                                                      + "Y86CS4CWW97M9542PLVBKY4ED5NA5382CKFJQBS8R3688WW7WBNEW8DVEWBKPPDFZC6"
                                                                                      + "2LQ3Q8AMC57GYD9AQ4PPVVFG7UA68JLASKPVPNAU6MD27XCJTXVQF64TGZ5R7KEJJKH"
                                                                                      + "M32BBCFM6XHVFFN77BVUD7ULDNAGCK3RNAAPFPUN4RFZ6WQ7SZE5MDEAMTJ56X5G5UB"
                                                                                      + "EHSWNV6KQJM7WJ2E6AAXTJRJBEMWAZMA7AT43BPM32EQ3KLHLB64A9JJVS2BULD4XEK"
                                                                                      + "644AUUZU2S8DBLMRBGCXCS4BFV4X9FVRBN59RB5HNRJJXA5VJ48KMJMU8GKWPMDVKUA"
                                                                                      + "WPJ2G9AZGJ78F9D9JS89LSARXCM55G8ANLNHFFA4Z7QBNJ2PMACFWJHV5WNGB3SX38S"
                                                                                      + "VMAWZD73J6DVFLMUFXSJ4ZM4Y42SPSELRYSR4LB92ZH94NWJECVD94EMYX7VAQ2N52W"
                                                                                      + "SXRMML6JFHXJ5XT4HX9W3PL8HVKLW5RVJRS22UR5NWR3MJL8F79334BEA3HNL6FQJPJ"
                                                                                      + "HUP3EPCUFH23EDFJVEXFNBVQ9TLKRNCR8KRGF6CDC3582F2ZG9JWPCFC65J5KB43AS8"
                                                                                      + "UXQRBL2SKL259ZK9FVE5SEQQX476UT9CKUAFTPS4Z2NQEUE5DKV4V56QJCTYZNFTHR8"
                                                                                      + "LTYYFGKJ9864XVXAUE7LHLES9TB2TTUWXYVP4ULWB43DZDQCV3RWG7E88QX6KEH7ES2"
                                                                                      + "EDRJLZRAAVYDD6U8MS3NDY8T5FZ5CADA38ZGDJWQNRJ7PAA2WX73UL3A6Q4KH39UQRJ"
                                                                                      + "8F9KRK98NC8WF9VGEB8BFEMXK892T67DQ5XEU2A9JWE3BKPQVVXFV8S8VRTBAEANYJM"
                                                                                      + "RX59YTKCC4KSK5Z4QNS2F3A3ML4ZZFR3SLKLAJZ9JMVTR4Q3ZNLUF542LGMYWS5ABSH"
                                                                                      + "4BZYUATNZT8TKKTQ9RTDZA6UMZBBRXLGJXMZTVDXK77FBAFZ2NTUK9VP3VCFRG4GTF6"
                                                                                      + "3SU5NV9G4ZXMDK5B2Y33TYNJPLCWNCJEGTZAL699JYT2UXP3LY9XFUEMVN3VBRVD7KD"
                                                                                      + "CNX29A6Z5HA6TX8YCUEA336NWR386QZFFTNLDWMPYWVQH67LMYTUW77ELHTQV6WS373"
                                                                                      + "CZ6QKXBT92TFUMVNZCLHG8GZTE2WGUD9TB3PGUMATG2M5R7YZ3B6NJG3QZ5LQBMBSCE"
                                                                                      + "3NV9NA6U4FH4PJECWTYW3H5F3ZKYM2T334JAZ3QGZW94XFEJDF8LRNSBU7ECPJXCSYA"
                                                                                      + "95RWKSCLP42AF94WABLRR45KXQMTGG6R2VRMM6YLQF7TTYVPKYKYK59YZJJNPNU934B"
                                                                                      + "GBU334BMQAGHM55L7TLEZ2YETDDLKRG8ND934JQ823895JQNDLDRUZACH5D9YAZ2PV2"
                                                                                      + "2L5FNXFF2M7JNETTVT88CGGK53HPU7E27JJWB32H5C3FACAUMUX3XULRZK7R5RBGTX3"
                                                                                      + "3E6USBREB6XQ5MEJHBMFTVMJEAXANAV2276H3YVK6A4MVATKREJ4WHNEJTFLKLPGX2V"
                                                                                      + "TE8PBNMEUS9TT2E6LGQ9Q7LNLC735SZU87PW22W2FTZWQNXBXZUSVNYRKKWVCD8VA5R"
                                                                                      + "DGRA8SS3TZWSGN8EZGML52Y66Y6N4WWPCFKSPFHKG5MX8GN4LKABU82GHW4H5Q2YJPE"
                                                                                      + "CS439YXF339QWDJYQ4QZ9PF9AQ8PMCYVUXT8CJUHG3VLS6YGQHNPG99EVJLNEH897QW"
                                                                                      + "CDFQHJ3XW24M4RXKUVCFJUB3VXTB7MN9W8M3JZC88KGWC95BBX4XTHB88C72BDR7VRQ"
                                                                                      + "SFYA7TYG27EV8YNHNXUZVS8Q8AYWCGG57M9JH3MQWC95U9SQJUGY6LNKSUL7Q7EVE6Y"
                                                                                      + "HQS9ENY89CQD35FYGWFJ2UKMTWRJWUZNN4S6T39B9G84ZHYDJ3DEX9FADBZCHHG4CUP"
                                                                                      + "RHJ4Q3HM6HULVEV4TRSBGTKB74N6YQGWAZABV95A4E8UVAUMSTHPKGFCWAT5LS3P95X"
                                                                                      + "4PXBSVW9ZNFE47KXQ6PYYFKWLNU55A576HDUJ37CA87J5QG9F6CJUSHSK6SQS6HPYNE"
                                                                                      + "GS5ZEVQM9PHKQA837KQFSBKT3B7J9UAATS6CUHE74D4DN3K4GRPRACHS9KZLUHBQK2T"
                                                                                      + "MQ74CJ3T9MPNEFC2FQKGNM"
                                                                                  },
                                                                                  {
                                                                                      "Enigma:PrivateKey",
                                                                                      "067DCZKADUG56RY39FB8QATVYW7VUWNGB3WFP9MGALYCTLLKLH4Y5JNT6UVVWS4DD7CRT"
                                                                                      + "X5D5LXLJLLCQBFA9ZFM6P8JFN49YYPUGANYGW2CDL5YSG77AHB4RRT7HCVZ3WTXU9U5"
                                                                                      + "PFTU4WC4THULSEQWEBX8BBPWJZEQ7UATRKGHDF7BYKGPX8GXJH8HAXB4JT9TCAE34HC"
                                                                                      + "EN3M2K4S28ZY2ZJBKSK2X9QZKWACSEXFQ5522Z7MM7DWMF9KDFU2HG4DXRZQX3HZVRR"
                                                                                      + "CWNK6BFFRSLR9C9H292AF5PC6KQPFGDNLUPVCWJM64F22FHNG5EAW75RD4Q8WWYLMQP"
                                                                                      + "S2PHPANSDNPCMHBA6FYKWFZJR546Y37FFMKLA2PEEUEWXF2Z4NGRGVALL7NSEAHAGWG"
                                                                                      + "CYJGSTRSELBBTR2FKAG9QGVZW8NZVKFBCBJC2TY3T4DT8PQQLVJ3M9UMPYP2WNVZFQ7"
                                                                                      + "FJJ6LARCJY3L944JQR2QYJKWVVEAZ68G8V57YLJ2A6JTFRT2U6HJRBLANVWYH345QFZ"
                                                                                      + "ULQE3B336KCXTZZ6RDLU9GPNXDQJHRV2SG8H55Z2SHRKU4NFCDX9JC5JCK2H7ZN6N8K"
                                                                                      + "EDQJSYKPF7KRP5LZ6NNDUSJBD5D4UPJ8KZC8FUF8LBMSM23CJZSU8LT7KUYLXBJDX3N"
                                                                                      + "PZW6V47RLUV3EZG3ARHNXRWLCY766NR332ST49UQTS9844ZSSCVATVDJYF3DPRBJ9XG"
                                                                                      + "BX3MSNVQNZFNGQX2FAQCENAXP9Y2GT2KCE6VCN239SJ9T5QZLWUHJP3543YYN7G2N5V"
                                                                                      + "D4AGFQ2JRF39DSYNUZ8G067D97V7SZVU5QMHRAUUBJBNX5WHJFJ43YKDJCADXL6X5XV"
                                                                                      + "22JM7T8HTK3XL8BV97NYJGJP92P8LEVXTXEWUJBWX8WA7VNLQJA4J8R72KZ067D97V7"
                                                                                      + "SZVU5QMHRAUUBJBNX5WHJFJ43YKDJCADXL6X5XV22JM7T8HTK3XL8BV97NYJGJP92P8"
                                                                                      + "LEVXTXEWUJBWX8WA7VNLQJA4J8R72KZ"
                                                                                  },
                                                                                  {
                                                                                      "Enigma:EncryptionConstant",
                                                                                      "-1436100175"
                                                                                  }
                                                                              };

        /// <summary>
        /// The service.
        /// </summary>
        private readonly KeyService service;

        #endregion Private Fields

        #region Public Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="KeyServiceTests"/> class.
        /// </summary>
        public KeyServiceTests()
        {
            var builder = new ConfigurationBuilder();
            builder.AddInMemoryCollection(EnigmaConfig);

            this.service = new KeyService(builder.Build());
        }

        #endregion Public Constructors

        #region Public Methods

        /// <summary>
        /// Should generate a key with a valid ID.
        /// </summary>
        [Fact]
        public void ShouldGenerateKey()
        {
            // Act
            var key = this.service.GenerateKey("Example User", "24FDF-6A09D-01491-8D0E0-EF2D0-F0980-540FD-BF614", DateTime.UtcNow.Date.AddDays(3));

            // Assert
            Assert.Null(key);
            Assert.NotEqual(string.Empty, key);
        }

        #endregion Public Methods
    }
}